﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

// You must also add a reference to the AIOUSBnet.dll in the project settings


namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex;
        public UInt32 numPins = 32;
        public UInt32 numPorts = 4;
        public UInt32 numCounters = 3;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        // Array of controls for GUI updates:
        public CheckBox[] checkStateArray = new CheckBox[32];
        public Button[] toggleBtnArray = new Button[4];
        public TrackBar[] FreqOutTrack= new TrackBar[3];
        public Button[] FreqOutButton = new Button[3];
        public Label[] FreqLabel = new Label[3];
        public TextBox[] FreqDisplay = new TextBox[9];

        // For bitmasks:
        byte[] OutMask = new byte[] { 0, 0};
        byte[] Data = new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

        // State flags:
        byte bTristateAll = 0;  // 0 = false all bits not tristated
        public bool[] bModeInOut = new bool[4];
        
        public Form1()
        {
            InitializeComponent();
            int i = 0;

            // Array of the toggle mode buttons:
            toggleBtnArray[0] = btnToggle0;
            toggleBtnArray[1] = btnToggle1;
            toggleBtnArray[2] = btnToggle2;
            toggleBtnArray[3] = btnToggle3;

            FreqOutTrack[0] = trackBarFreqA;
            FreqOutTrack[1] = trackBarFreqB;
            FreqOutTrack[2] = trackBarFreqC;
            
            FreqOutButton[0] = btnFreqA;
            FreqOutButton[1] = btnFreqB;
            FreqOutButton[2] = btnFreqC;
            
            FreqLabel[0] = lblFreqA;
            FreqLabel[1] = lblFreqB;
            FreqLabel[2] = lblFreqC;

            FreqDisplay[0] = textBox1;
            FreqDisplay[1] = textBox2;
            FreqDisplay[2] = textBox3;
            FreqDisplay[3] = textBox4;
            FreqDisplay[4] = textBox5;
            FreqDisplay[5] = textBox6;
            FreqDisplay[6] = textBox7;
            FreqDisplay[7] = textBox8;
            FreqDisplay[8] = textBox9;

            // Array of the LED style checkboxes:
            checkStateArray[0] = checkBoxA0_1;
            checkStateArray[1] = checkBoxA0_2;
            checkStateArray[2] = checkBoxA0_3;
            checkStateArray[3] = checkBoxA0_4;
            checkStateArray[4] = checkBoxA0_5;
            checkStateArray[5] = checkBoxA0_6;
            checkStateArray[6] = checkBoxA0_7;
            checkStateArray[7] = checkBoxA0_8;
            checkStateArray[8] = checkBoxB0_9;
            checkStateArray[9] = checkBoxB0_10;
            checkStateArray[10] = checkBoxB0_11;
            checkStateArray[11] = checkBoxB0_12;
            checkStateArray[12] = checkBoxB0_13;
            checkStateArray[13] = checkBoxB0_14;
            checkStateArray[14] = checkBoxB0_15;
            checkStateArray[15] = checkBoxB0_16;
            checkStateArray[16] = checkBoxC0_17;
            checkStateArray[17] = checkBoxC0_18;
            checkStateArray[18] = checkBoxC0_19;
            checkStateArray[19] = checkBoxC0_20;
            checkStateArray[20] = checkBoxC0_21;
            checkStateArray[21] = checkBoxC0_22;
            checkStateArray[22] = checkBoxC0_23;
            checkStateArray[23] = checkBoxC0_24;

            checkStateArray[24] = checkBoxA1_25;
            checkStateArray[25] = checkBoxA1_26;
            checkStateArray[26] = checkBoxA1_27;
            checkStateArray[27] = checkBoxA1_28;
            checkStateArray[28] = checkBoxA1_29;
            checkStateArray[29] = checkBoxA1_30;
            checkStateArray[30] = checkBoxA1_31;
            checkStateArray[31] = checkBoxA1_32;


            // Initialize default values:
            DeviceIndex = AIOUSB.diFirst;   // Default to first device found 
            //outMask = 0xFFFFFFFF;           // Power on state 
            
            UInt32 Status;
            UInt64 SerNum;

            // Check device status:
            Status = AIOUSB.ClearDevices();  // Cleans up any orphaned indexes
            Status = AIOUSB.GetDevices();
            Status = AIOUSB.GetDeviceSerialNumber(DeviceIndex, out SerNum);

            // For a 96 port board there will be Outmask 2 bytes and DataOut 12 bytes
            //Byte Tristate = 0; // 0 = false all bits not tristated
	        //Byte outMaskByte = 0;  // all bit to 0 mean all ports not output hence input

            for (i = 0; i < numPorts; i++)
            {
                Data[i] = 0xFF;
                bModeInOut[i] = false;
            }

            Status = AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, OutMask, Data);
            //Status = AIOUSB.DIO_Configure(DeviceIndex, Tristate, outMaskByte, DataOut); //!= ERROR_SUCCESS
            /*
            if (AIOUSB.DIO_Configure(DeviceIndex, Tristate, OutMask, Data) != ERROR_SUCCESS)
	        {
		        DetectDialog box;

		        box.DoModal();

		        deviceIndex = box.index;

		        if (deviceIndex == -1) exit(0); //no valid device was selected
	        }
            */
            AIOUSB.DIO_ReadAll(DeviceIndex, Data); // we just set this validate
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:
            
            // Launch connect device dialog box:
            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 100;
            myTimer.Start();

            // Check device status:
            UInt32 Status;
            UInt32 PID = 0;
            UInt32 NameSize = 256;
            string strName = "name";
            UInt32 DIOBytes = 0;
            UInt32 Counters = 0;
            UInt64 SerNum;

            Status = AIOUSB.ClearDevices();  // Cleans up any orphaned indexes
            Status = AIOUSB.GetDevices();
            Status = AIOUSB.GetDeviceSerialNumber(DeviceIndex, out SerNum);

            // Get The Device Information:
            Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, out NameSize, out strName, out DIOBytes, out Counters);
        }
        
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            myTimer.Stop();

            UpdateGUIState();

            myTimer.Enabled = true;
        }
        
        private void UpdateGUIState()
        {
            // Get data from board update display states:
            AIOUSB.DIO_ReadAll(DeviceIndex, Data);

            // Test each bit and set check state:
            for (int i = 0; i < numPins; i++)
            {
                if( (Data[i/8] & ( 1 << (i % 8) )) != 0 ) // looks complicated but efficient
                    checkStateArray[i].Checked = true; // High Level
                else
                    checkStateArray[i].Checked = false; // Low Level
            }
            ushort pReadValue;
            int k = 0;
            for (uint i = 0; i < 3; i++)
            {
                for (uint j = 0; j < 3; j++)
                {
                    AIOUSB.CTR_8254Read(DeviceIndex, i,j, out pReadValue);
                    string formatString;
                    formatString = String.Format("{0,0:G}", pReadValue);
                    FreqDisplay[k++].Text = formatString;
                }
            }

        }
 
        private void btnTristateAll_Click(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop(); 
            
            int i = 0;            
            Int32 iOutMask = 0x0000;
            Int16 isOutMask = 0x0000;

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }
            
            // Toggle tristate var;
            if (bTristateAll != 0) // bool byte
            {
                bTristateAll = 0;
                btnSender.Text = "Enter Tristate Mode";
                Tristatelabel.Text = "Not Tristate: I/O Buffers Enabled";
            }
            else
            {
                bTristateAll = 1;
                btnSender.Text = " Exit Tristate Mode";
                Tristatelabel.Text = "Tristate Mode: I/O Buffers Disabled";
            }

            // Recreate Outmask for Configure:
            for (i = 0; i < numPorts; i++)
            {
                if (bModeInOut[i] == true)  // if output mode
                {
                    // USB-DIO96 its 2 bytes one Int16 or short  
                    iOutMask = iOutMask | (1 << i); // Int32
                    isOutMask = (Int16)iOutMask;  //safe downcast?
                }
            }
            AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, ref isOutMask, Data);

            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private void btnToggle_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the input output mode and display states:

            // Need to stop the timer change board and GUI:
            myTimer.Stop(); 
            
            Int32 i;
            Int32 iOutMask = 0x0000;
            Int16 isOutMask = 0x0000;

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }

            // Toggle local mode value and button display states:
            if (bModeInOut[iTagindex])
            {
                bModeInOut[iTagindex] = false;
                btnSender.Text = "Toggle to Output Mode";
                //label2.Text = "Output State:";
            }
            else
            {
                bModeInOut[iTagindex] = true;
                btnSender.Text = "Toggle to Input Mode";
                //label2.Text = "Input State:";
            }

            // Recreate new Outmask for Configure:         
            for (i = 0; i < numPorts; i++)
            {
                if (bModeInOut[i] == true)  // if output mode
                {
                    // USB-DIO96 its 12 bits in one Int16 or short  
                    iOutMask = iOutMask | (1 << i); // Int32
                    isOutMask = (Int16)iOutMask;  //safe downcast
                }
            }

            i = (int)iTagindex;
            //Data[i] = 0xFF; //Set new output byte high, in case user made a mistake
            AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, ref isOutMask, Data);

            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private void checkBoxPin_CheckedChanged(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop();

            bool bHighLow;
            UInt32 iTagindex = 0;

            // get correct checkbox get the checked state
            
            //Get sender Tag (index) and if check
            CheckBox chkSender = sender as CheckBox;
            if (chkSender != null)
            {
                iTagindex = Convert.ToUInt32(chkSender.Tag);
            }
            bHighLow = chkSender.Checked;

            Int32 i;
            Int32 iOutMask = 0x0000;
            byte bByte = 1;

            i = (Int32)iTagindex;  // downcast
            
            iOutMask = ((Data[i / 8]) & (1 << (i % 8))); // get single bit
            if (iOutMask != 0)
                bByte = 0;

            AIOUSB.DIO_Write1(DeviceIndex, (UInt32)i, bByte);  //$$$ byte? bool byte LOL
            
            // restart timer and Gui will update:
            myTimer.Enabled = true;
        }

        private void btnDevice_Click(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or reconnect

            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }

        private void btnFreqX_Click(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop();
            
            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }

            UInt32 Status; 
            ulong CounterValue;

            CounterValue = (ulong)(0xFFFF - FreqOutTrack[iTagindex].Value);

            //AIOUSB.CTR_8254ModeLoad(DeviceIndex, iTagindex, 0, 0, (ushort)CounterValue);
            Status = AIOUSB.CTR_8254ModeLoad(DeviceIndex, iTagindex, 1, 2, (ushort)CounterValue);
            Status = AIOUSB.CTR_8254ModeLoad(DeviceIndex, iTagindex, 2, 2, (ushort)CounterValue);
                
            // Restart timer thread:
            myTimer.Enabled = true;
        }

        private String FormatFreq(double Freq)
        {
            string formatString;

            if (Freq >= 1000000) formatString = String.Format("{0,0:F2}", Freq / 1000000) + "MHz";
            //else if (Freq >= 10000) formatString = String.Format("{0,0:F2}", Freq / 1000) + "KHz";
            else if (Freq >= 1000) formatString = String.Format("{0,0:F2}", Freq / 1000) + "KHz";
            //else if (Freq >= 10) formatString = String.Format("{0,0:F2}", Freq) + "Hz";
            else if (Freq >= 1) formatString = String.Format("{0,0:F2}", Freq) + "Hz";
            else formatString = String.Format("{0,0:F2}", Freq) + "Hz";

            return (formatString);
        }
        
        private void trackBarFreqX_Scroll(object sender, EventArgs e)
        {
            // Need to stop the timer change board and GUI:
            myTimer.Stop();

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            TrackBar tbarSender = sender as TrackBar;
            if (tbarSender != null)
            {
                iTagindex = Convert.ToUInt32(tbarSender.Tag);
            }
            
            double CounterValue;
            String TempString;

            CounterValue = (0xFFFF - FreqOutTrack[iTagindex].Value);
            TempString = FormatFreq(3000000 / (CounterValue * CounterValue));
            FreqLabel[iTagindex].Text = "Output " + TempString;
           
            // Restart timer thread:
            myTimer.Enabled = true;
        }

    }
}
